-- Create table to hold survey
DROP TABLE SURVEY CASCADE CONSTRAINTS ; 
CREATE TABLE SURVEY ( 
  SURVEY_ID    NUMBER        NOT NULL, 
  SURVEY_NAME  VARCHAR2(500), 
  SURVEY_DESC  VARCHAR2(2000), 
  START_DATE   DATE, 
  END_DATE     DATE, 
  ATTRIBUTE1   VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE2   VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE3   VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE4   VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE5   VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE6   VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE7   VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE8   VARCHAR2(250)  DEFAULT null, 
  CONSTRAINT SURVEY_PK
  PRIMARY KEY ( SURVEY_ID ) ) ; 

--Create table to hold questions
DROP TABLE QUESTIONS CASCADE CONSTRAINTS ; 
CREATE TABLE QUESTIONS ( 
  QUESTION_ID      NUMBER        NOT NULL, 
  SURVEY_ID        NUMBER, 
  QUESTION_NUM     NUMBER, 
  QUESTION_TEXT    VARCHAR2(2000), 
  QUESTION_FORMAT  NUMBER, 
  ATTRIBUTE1       VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE2       VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE3       VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE4       VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE5       VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE6       VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE7       VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE8       VARCHAR2(250)  DEFAULT null, 
  CONSTRAINT QUESTION_PK
  PRIMARY KEY ( QUESTION_ID ) ) ; 

-- Create Questions to hold answers to each question
DROP TABLE ANSWERS CASCADE CONSTRAINTS ; 
CREATE TABLE ANSWERS ( 
  ANSWER_ID         NUMBER        NOT NULL, 
  QUESTION_ID       NUMBER, 
  ANSWER_TEXT       VARCHAR2(50), 
  ANSWER_ORDER      NUMBER, 
  DEFAULT_ANS_FLAG  CHAR(1)       DEFAULT 'N', 
  ATTRIBUTE1        VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE2        VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE3        VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE4        VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE5        VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE6        VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE7        VARCHAR2(250)  DEFAULT null, 
  ATTRIBUTE8        VARCHAR2(250)  DEFAULT null, 
  CONSTRAINT ANSWER_PK
  PRIMARY KEY ( ANSWER_ID ) ) ; 

-- Create table to hold user responses
DROP TABLE RESPONSES CASCADE CONSTRAINTS ; 
CREATE TABLE RESPONSES ( 
  RESPONSE_ID   NUMBER        NOT NULL, 
  QUESTION_ID   NUMBER, 
  ANSWER_ID     NUMBER, 
  DATE_CREATED  DATE          DEFAULT sysdate, 
  CONSTRAINT RESPONSE_PK
  PRIMARY KEY ( RESPONSE_ID ) ) ; 

-- Make answers and responses accessible
-- This may be necessary to get the permissions correct on the
-- calls to owa_util.list_print
GRANT SELECT ON ANSWERS TO PUBLIC;
GRANT SELECT ON RESPONSES TO PUBLIC;

-- Create sequences used for PKs
drop sequence survey_seq;
create sequence survey_seq start with 1;

drop sequence question_seq;
create sequence question_seq start with 1;

drop sequence answer_seq;
create sequence answer_seq start with 1;

drop sequence response_seq;
create sequence response_seq start with 1;

-- Populate test data
insert into survey (survey_id, survey_name, survey_desc) values
   (survey_seq.nextval, 'Test', 'Test survery data');
   
insert into questions (question_id, survey_id, question_num, question_text) values
   (question_seq.nextval, 1, 1, 'Test question 1');
insert into questions (question_id, survey_id, question_num, question_text) values
   (question_seq.nextval, 1, 2, 'Test question 2');


insert into answers (answer_id, question_id, answer_text) values
   (answer_seq.nextval, 1, 'Q1, Test answer 1');   
insert into answers (answer_id, question_id, answer_text) values
   (answer_seq.nextval, 1, 'Q1, Test answer 2');      
insert into answers (answer_id, question_id, answer_text) values
   (answer_seq.nextval, 2, 'Q2, Test answer 1');
insert into answers (answer_id, question_id, answer_text) values
   (answer_seq.nextval, 2, 'Q2, Test answer 2');            
   
commit;

-- Create necessary views
drop view survey_results;
create view survey_results as
   select
      s.survey_id,
      s.survey_name "SURVEY_NAME",
      q.question_id,
      q.question_text "QUESTION_TEXT",
      a.answer_id,
      a.answer_text "ANSWER_TEXT",
      count(r.answer_id) "NUM_RESPONSES"
   from
      survey s,
      questions q,
      answers a,
      responses r
   where
      s.survey_id = q.survey_id and
      q.question_id = a.question_id and
      a.answer_id = r.answer_id (+)
   group by
      s.survey_id,
      s.survey_name,
      q.question_id,
      q.question_text,
      a.answer_id,
      a.answer_text;
	  
drop view survey_results_view;
create view survey_results_view as
   select
      s.survey_id,
      s.survey_name "SURVEY_NAME",
      q.question_id,
      q.question_text "QUESTION_TEXT",
      a.answer_id,
      a.answer_text "ANSWER_TEXT",
      count(r.answer_id) "NUM_RESPONSES"
   from
      survey s,
      questions q,
      answers a,
      responses r
   where
      s.survey_id = q.survey_id and
      q.question_id = a.question_id and
      a.answer_id = r.answer_id (+)
   group by
      s.survey_id,
      s.survey_name,
      q.question_id,
      q.question_text,
      a.answer_id,
      a.answer_text;
	  

